@@ -25,11 +25,11 @@ from weakref import WeakKeyDictionary
 from hypothesis import settings as Settings
 from hypothesis import Phase
 from hypothesis.reporting import debug_report
-from hypothesis.internal.compat import EMPTY_BYTES, Counter, hbytes, \
-    hrange, text_type, bytes_from_list, to_bytes_sequence, \
+from hypothesis.internal.compat import EMPTY_BYTES, Counter, ceil, \
+    hbytes, hrange, text_type, bytes_from_list, to_bytes_sequence, \
     unicode_safe_repr
-from hypothesis.internal.conjecture.data import Status, StopTest, \
-    ConjectureData
+from hypothesis.internal.conjecture.data import MAX_DEPTH, Status, \
+    StopTest, ConjectureData
 from hypothesis.internal.conjecture.minimizer import minimize
 
 
@@ -77,16 +77,22 @@ class ConjectureRunner(object):
         # Recursively, a node is dead if either it is a leaf or every byte
         # leads to a dead node when starting from here.
         self.dead = set()
+        self.forced = {}
 
     def __tree_is_exhausted(self):
         return 0 in self.dead
 
     def new_buffer(self):
         assert not self.__tree_is_exhausted()
+
+        def draw_bytes(data, n, distribution):
+            return self.__rewrite_for_novelty(
+                data, self.__zero_bound(data, distribution(self.random, n))
+            )
+
         self.last_data = ConjectureData(
             max_length=self.settings.buffer_size,
-            draw_bytes=lambda data, n, distribution:
-            self.__rewrite_for_novelty(data, distribution(self.random, n))
+            draw_bytes=draw_bytes
         )
         self.test_function(self.last_data)
         self.last_data.freeze()
@@ -113,25 +119,27 @@ class ConjectureRunner(object):
 
         tree_node = self.tree[0]
         indices = []
-        i = 0
-        for b in data.buffer:
-            indices.append(i)
+        node_index = 0
+        for i, b in enumerate(data.buffer):
+            indices.append(node_index)
+            if i in data.forced_indices:
+                self.forced[node_index] = b
             try:
-                i = tree_node[b]
+                node_index = tree_node[b]
             except KeyError:
-                i = len(self.tree)
+                node_index = len(self.tree)
                 self.tree.append({})
-                tree_node[b] = i
-            tree_node = self.tree[i]
-            if i in self.dead:
+                tree_node[b] = node_index
+            tree_node = self.tree[node_index]
+            if node_index in self.dead:
                 break
 
-        if data.status != Status.OVERRUN and i not in self.dead:
-            self.dead.add(i)
-            self.tree[i] = data
+        if data.status != Status.OVERRUN and node_index not in self.dead:
+            self.dead.add(node_index)
+            self.tree[node_index] = data
 
             for j in reversed(indices):
-                if len(self.tree[j]) < 256:
+                if len(self.tree[j]) < 256 and j not in self.forced:
                     break
                 if set(self.tree[j].values()).issubset(self.dead):
                     self.dead.add(j)
@@ -157,19 +165,16 @@ class ConjectureRunner(object):
         if data.status == Status.INTERESTING:
             assert len(data.buffer) <= len(self.last_data.buffer)
             if len(data.buffer) == len(self.last_data.buffer):
-                assert data.buffer < self.last_data.buffer
+                return data.buffer < self.last_data.buffer
             return True
         return True
 
     def save_buffer(self, buffer):
         if (
             self.settings.database is not None and
-            self.database_key is not None and
-            Phase.reuse in self.settings.phases
+            self.database_key is not None
         ):
-            self.settings.database.save(
-                self.database_key, hbytes(buffer)
-            )
+            self.settings.database.save(self.database_key, hbytes(buffer))
 
     def note_details(self, data):
         if data.status == Status.INTERESTING:
@@ -191,6 +196,22 @@ class ConjectureRunner(object):
             data.output,
         ))
 
+    def prescreen_buffer(self, buffer):
+        i = 0
+        for b in buffer:
+            if i in self.dead:
+                return False
+            try:
+                b = self.forced[i]
+            except KeyError:
+                pass
+            try:
+                i = self.tree[i][b]
+            except KeyError:
+                return True
+        else:
+            return False
+
     def incorporate_new_buffer(self, buffer):
         assert self.last_data.status == Status.INTERESTING
         if (
@@ -200,19 +221,11 @@ class ConjectureRunner(object):
             self.exit_reason = ExitReason.timeout
             raise RunIsComplete()
 
-        buffer = buffer[:self.last_data.index]
+        buffer = hbytes(buffer[:self.last_data.index])
         if sort_key(buffer) >= sort_key(self.last_data.buffer):
             return False
 
-        i = 0
-        for b in buffer:
-            if i in self.dead:
-                return False
-            try:
-                i = self.tree[i][b]
-            except KeyError:
-                break
-        else:
+        if not self.prescreen_buffer(buffer):
             return False
 
         assert sort_key(buffer) <= sort_key(self.last_data.buffer)
@@ -312,25 +325,68 @@ class ConjectureRunner(object):
             else:
                 result = self.random.choice(bits)(data, n, distribution)
 
-            return self.__rewrite_for_novelty(data, result)
+            return self.__rewrite_for_novelty(
+                data, self.__zero_bound(data, result))
 
         return draw_mutated
 
+    def __rewrite(self, data, result):
+        return self.__rewrite_for_novelty(
+            data, self.__zero_bound(data, result)
+        )
+
+    def __zero_bound(self, data, result):
+        """This tries to get the size of the generated data under control by
+        replacing the result with zero if we are too deep or have already
+        generated too much data.
+
+        This causes us to enter "shrinking mode" there and thus reduce
+        the size of the generated data.
+
+        """
+        if (
+            data.depth * 2 >= MAX_DEPTH or
+            (data.index + len(result)) * 2 >= self.settings.buffer_size
+        ):
+            if any(result):
+                data.hit_zero_bound = True
+            return hbytes(len(result))
+        else:
+            return result
+
     def __rewrite_for_novelty(self, data, result):
+        """Take a block that is about to be added to data as the result of a
+        draw_bytes call and rewrite it a small amount to ensure that the result
+        will be novel: that is, not hit a part of the tree that we have fully
+        explored.
+
+        This is mostly useful for test functions which draw a small
+        number of blocks.
+
+        """
         assert isinstance(result, hbytes)
         try:
             node_index = data.__current_node_index
         except AttributeError:
-            assert len(data.buffer) == 0
             node_index = 0
             data.__current_node_index = node_index
             data.__hit_novelty = False
+            data.__evaluated_to = 0
 
         if data.__hit_novelty:
             return result
 
         node = self.tree[node_index]
-        assert node_index not in self.dead
+
+        for i in hrange(data.__evaluated_to, len(data.buffer)):
+            node = self.tree[node_index]
+            try:
+                node_index = node[data.buffer[i]]
+                assert node_index not in self.dead
+                node = self.tree[node_index]
+            except KeyError:
+                data.__hit_novelty = True
+                return result
 
         for i, b in enumerate(result):
             assert isinstance(b, int)
@@ -364,51 +420,73 @@ class ConjectureRunner(object):
             node = new_node
         assert node_index not in self.dead
         data.__current_node_index = node_index
+        data.__evaluated_to = data.index + len(result)
         return hbytes(result)
 
+    def has_existing_examples(self):
+        return (
+            self.settings.database is not None and
+            self.database_key is not None and
+            Phase.reuse in self.settings.phases
+        )
+
+    def reuse_existing_examples(self):
+        """If appropriate (we have a database and have been told to use it),
+        try to reload existing examples from the database.
+
+        If there are a lot we don't try all of them. We always try the
+        smallest example in the database (which is guaranteed to be the
+        last failure) and the largest (which is usually the seed example
+        which the last failure came from but we don't enforce that). We
+        then take a random sampling of the remainder and try those. Any
+        examples that are no longer interesting are cleared out.
+
+        """
+        if self.has_existing_examples():
+            corpus = sorted(
+                self.settings.database.fetch(self.database_key),
+                key=sort_key
+            )
+
+            desired_size = max(2, ceil(0.1 * self.settings.max_examples))
+
+            if desired_size < len(corpus):
+                new_corpus = [corpus[0], corpus[-1]]
+                n_boost = max(desired_size - 2, 0)
+                new_corpus.extend(self.random.sample(corpus[1:-1], n_boost))
+                corpus = new_corpus
+                corpus.sort(key=sort_key)
+
+            for existing in corpus:
+                if self.valid_examples >= self.settings.max_examples:
+                    self.exit_with(ExitReason.max_examples)
+                if self.call_count >= max(
+                    self.settings.max_iterations, self.settings.max_examples
+                ):
+                    self.exit_with(ExitReason.max_iterations)
+                data = ConjectureData.for_buffer(existing)
+                self.test_function(data)
+                data.freeze()
+                self.last_data = data
+                self.consider_new_test_data(data)
+                if data.status == Status.INTERESTING:
+                    assert data.status == Status.INTERESTING
+                    self.last_data = data
+                    break
+                else:
+                    self.settings.database.delete(
+                        self.database_key, existing)
+
+    def exit_with(self, reason):
+        self.exit_reason = reason
+        raise RunIsComplete()
+
     def _run(self):
         self.last_data = None
         mutations = 0
         start_time = time.time()
 
-        if (
-            self.settings.database is not None and
-            self.database_key is not None
-        ):
-            corpus = sorted(
-                self.settings.database.fetch(self.database_key),
-                key=lambda d: (len(d), d)
-            )
-            for existing in corpus:
-                if self.valid_examples >= self.settings.max_examples:
-                    self.exit_reason = ExitReason.max_examples
-                    return
-                if self.call_count >= max(
-                    self.settings.max_iterations, self.settings.max_examples
-                ):
-                    self.exit_reason = ExitReason.max_iterations
-                    return
-                data = ConjectureData.for_buffer(existing)
-                self.test_function(data)
-                data.freeze()
-                self.last_data = data
-                self.consider_new_test_data(data)
-                if data.status < Status.VALID:
-                    self.settings.database.delete(
-                        self.database_key, existing)
-                elif data.status == Status.VALID:
-                    # Incremental garbage collection! we store a lot of
-                    # examples in the DB as we shrink: Those that stay
-                    # interesting get kept, those that become invalid get
-                    # dropped, but those that are merely valid gradually go
-                    # away over time.
-                    if self.random.randint(0, 2) == 0:
-                        self.settings.database.delete(
-                            self.database_key, existing)
-                else:
-                    assert data.status == Status.INTERESTING
-                    self.last_data = data
-                    break
+        self.reuse_existing_examples()
 
         if (
             Phase.generate in self.settings.phases and not
@@ -421,6 +499,9 @@ class ConjectureRunner(object):
                 self.new_buffer()
 
             mutator = self._new_mutator()
+
+            zero_bound_queue = []
+
             while (
                 self.last_data.status != Status.INTERESTING and
                 not self.__tree_is_exhausted()
@@ -439,9 +520,41 @@ class ConjectureRunner(object):
                 ):
                     self.exit_reason = ExitReason.timeout
                     return
-                if mutations >= self.settings.max_mutations:
+                if zero_bound_queue:
+                    # Whenever we generated an example and it hits a bound
+                    # which forces zero blocks into it, this creates a weird
+                    # distortion effect by making certain parts of the data
+                    # stream (especially ones to the right) much more likely
+                    # to be zero. We fix this by redistributing the generated
+                    # data by shuffling it randomly. This results in the
+                    # zero data being spread evenly throughout the buffer.
+                    # Hopefully the shrinking this causes will cause us to
+                    # naturally fail to hit the bound.
+                    # If it doesn't then we will queue the new version up again
+                    # (now with more zeros) and try again.
+                    overdrawn = zero_bound_queue.pop()
+                    buffer = bytearray(overdrawn.buffer)
+                    self.random.shuffle(buffer)
+                    buffer = hbytes(buffer)
+
+                    if buffer == overdrawn.buffer:
+                        continue
+
+                    def draw_bytes(data, n, distribution):
+                        result = buffer[data.index:data.index + n]
+                        if len(result) < n:
+                            result += hbytes(n - len(result))
+                        return self.__rewrite(data, result)
+
+                    data = ConjectureData(
+                        draw_bytes=draw_bytes,
+                        max_length=self.settings.buffer_size,
+                    )
+                    self.test_function(data)
+                    data.freeze()
+                elif mutations >= self.settings.max_mutations:
                     mutations = 0
-                    self.new_buffer()
+                    data = self.new_buffer()
                     mutator = self._new_mutator()
                 else:
                     data = ConjectureData(
@@ -457,7 +570,8 @@ class ConjectureRunner(object):
                             mutations = 0
                     else:
                         mutator = self._new_mutator()
-
+                if getattr(data, 'hit_zero_bound', False):
+                    zero_bound_queue.append(data)
                 mutations += 1
 
         if self.__tree_is_exhausted():
@@ -563,6 +677,22 @@ class ConjectureRunner(object):
             self.exit_reason = ExitReason.finished
             return
 
+        if self.has_existing_examples():
+            corpus = sorted(
+                self.settings.database.fetch(self.database_key),
+                key=sort_key
+            )
+            # We always have self.last_data.buffer in the database because
+            # we save every interesting example. This means we will always
+            # trigger the first break and thus never exit the loop normally.
+            for c in corpus:  # pragma: no branch
+                if sort_key(c) >= sort_key(self.last_data.buffer):
+                    break
+                elif self.incorporate_new_buffer(c):
+                    break
+                else:
+                    self.settings.database.delete(self.database_key, c)
+
         change_counter = -1
 
         while self.changed > change_counter:
